/**
 * @file libgalago/galago-dbus.h D-BUS Support API
 *
 * @Copyright (C) 2004-2005 Christian Hammond.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _GALAGO_DBUS_H_
#define _GALAGO_DBUS_H_

#define DBUS_API_SUBJECT_TO_CHANGE

#include <dbus/dbus.h>
#include <libgalago/galago-list.h>
#include <libgalago/galago-object.h>

#define GALAGO_DBUS_SERVICE "org.freedesktop.Galago"


/**************************************************************************
 * Utility functions
 **************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif

/**
 * Appends a string to a message, or a NIL if the string is NULL.
 *
 * @param iter The message iterator.
 * @param str  The string, or NULL.
 */
void galago_dbus_message_iter_append_string_or_nil(DBusMessageIter *iter,
												   const char *str);

/**
 * Returns a string that the iterator points to.
 *
 * If the returned type is a NIL, this will return a NULL.
 *
 * @param iter The iterator.
 *
 * @return The string or NULL.
 */
char *galago_dbus_message_iter_get_string_or_nil(DBusMessageIter *iter);

/**
 * Appends an object to a D-BUS message.
 *
 * @param iter   The D-BUS message iterator.
 * @param object The object to append.
 */
void galago_dbus_message_iter_append_object(DBusMessageIter *iter,
											const void *object);

/**
 * Appends a list of objects to a D-BUS message.
 *
 * @param iter The D-BUS message iterator.
 * @param list The list of objects to append.
 */
void galago_dbus_message_iter_append_object_list(DBusMessageIter *iter,
												 const GalagoList *list);

/**
 * Returns an object of the specified type from a D-BUS message.
 *
 * @param iter  The D-BUS message iterator.
 * @param klass The object class.
 *
 * @return The object.
 */
void *galago_dbus_message_iter_get_object(DBusMessageIter *iter,
										  const GalagoObjectClass *klass);

/**
 * Returns a list of objects of the specified type from a D-BUS message.
 *
 * The returned list must be destroyed.
 *
 * @param iter  The D-BUS message iterator.
 * @param klass The object class.
 *
 * @return The list.
 */
GalagoList *galago_dbus_message_iter_get_object_list(
	DBusMessageIter *iter, const GalagoObjectClass *klass);

/**
 * Creates a D-BUS message to the specified object.
 *
 * @param object   The object.
 * @param name     The message name.
 * @param reply    TRUE if a reply is expected.
 * @param ret_iter The returned message iterator, if not NULL.
 *
 * @return The D-BUS message.
 */
DBusMessage *galago_dbus_message_new_method_call(const void *object,
												 const char *name,
												 galago_bool reply,
												 DBusMessageIter *ret_iter);

/**
 * Pushes an object and all its internal data to the server.
 *
 * This should only be used internally.
 *
 * @param object The object to push.
 */
void galago_dbus_object_push_full(void *object);

#ifdef __cplusplus
}
#endif

#endif /* _GALAGO_DBUS_H_ */
